diff --git a/src/plugins/debugger/registerhandler.cpp b/src/plugins/debugger/registerhandler.cpp
index 6419bbf2b8acb292ef8a1b1df51c2e9de4eba8d3..cbff586239b887b775cdbb84914e19062a9994ea 100644
--- a/src/plugins/debugger/registerhandler.cpp
+++ b/src/plugins/debugger/registerhandler.cpp
@@ -67,8 +67,6 @@ int RegisterHandler::columnCount(const QModelIndex &parent) const
 
 QVariant RegisterHandler::data(const QModelIndex &index, int role) const
 {
-    static const QVariant red = QColor(200, 0, 0);
-
     if (role == RegisterNumberBaseRole)
         return m_base;
 
@@ -78,7 +76,7 @@ QVariant RegisterHandler::data(const QModelIndex &index, int role) const
     const Register &reg = m_registers.at(index.row());
 
     if (role == RegisterAddressRole) {
-        // return some address associated with the register
+        // Return some address associated with the register.
         bool ok = true;
         qulonglong value = reg.value.toULongLong(&ok, 0);
         return ok ? QString::fromLatin1("0x") + QString::number(value, 16) : QVariant();
@@ -100,8 +98,8 @@ QVariant RegisterHandler::data(const QModelIndex &index, int role) const
     if (role == Qt::TextAlignmentRole && index.column() == 1)
         return Qt::AlignRight;
 
-    if (role == Qt::TextColorRole && reg.changed && index.column() == 1)
-        return red;
+    if (role == RegisterChangedRole)
+        return reg.changed;
     
     return QVariant();
 }
diff --git a/src/plugins/debugger/registerwindow.cpp b/src/plugins/debugger/registerwindow.cpp
index 51fb69ff52248d8af6bf74d7b14ed0592e7ebdea..13c7a537c2aa289452c07835faf6b571e90a4734 100644
--- a/src/plugins/debugger/registerwindow.cpp
+++ b/src/plugins/debugger/registerwindow.cpp
@@ -105,6 +105,10 @@ public:
         const QModelIndex &index) const
     {
         if (index.column() == 1) {
+            bool paintRed = index.data(RegisterChangedRole).toBool();
+            QPen oldPen = painter->pen();
+            if (paintRed)
+                painter->setPen(QColor(200, 0, 0));
             // FIXME: performance? this changes only on real font changes.
             QFontMetrics fm(option.font);
             int charWidth = fm.width(QLatin1Char('x'));
@@ -112,7 +116,7 @@ public:
                 charWidth = qMax(charWidth, fm.width(QLatin1Char(i)));
             for (int i = 'a'; i <= 'f'; ++i)
                 charWidth = qMax(charWidth, fm.width(QLatin1Char(i)));
-            QString str = index.model()->data(index, Qt::DisplayRole).toString();
+            QString str = index.data(Qt::DisplayRole).toString();
             int x = option.rect.x();
             for (int i = 0; i < str.size(); ++i) {
                 QRect r = option.rect;
@@ -121,6 +125,8 @@ public:
                 x += charWidth;
                 painter->drawText(r, Qt::AlignHCenter, QString(str.at(i)));
             }
+            if (paintRed)
+                painter->setPen(oldPen);
         } else {
             QItemDelegate::paint(painter, option, index);
         }
diff --git a/src/plugins/debugger/registerhandler.h b/src/plugins/debugger/registerhandler.h
index c2766dd64665fa25511f7f0c64c3ceb9488d2807..d7f16d04637b4eac4a199b6a58a1cbebb69a114d 100644
--- a/src/plugins/debugger/registerhandler.h
+++ b/src/plugins/debugger/registerhandler.h
@@ -38,7 +38,8 @@ namespace Internal {
 enum RegisterRole
 {
     RegisterNumberBaseRole = Qt::UserRole, // Currently used number base
-    RegisterAddressRole                   // Start value for opening memory view
+    RegisterAddressRole,                   // Start value for opening memory view
+    RegisterChangedRole                    // Used for painting changed values 
 };
 
 class Register